package org.trinityhealth.example.canon;

import java.util.HashMap;

import org.apache.camel.Exchange;
import org.apache.camel.Processor;
import org.apache.camel.builder.RouteBuilder;

public class CanonRoute extends RouteBuilder {
	public CanonRoute() {
		super();
	}
	
	public void configure() {
        from("cxfrs:bean:canonService?bindingStyle=SimpleConsumer")
//        from("direct:fakeCXF")
        .log("RECEIVED HTTP REQUEST, QUEUING UP '" + header("CamelHttpMethod") + "'")
        .to("direct:incomingRequestQueue");
        
		from("direct:incomingRequestQueue")
		.log("REQUEST ARRIVED ON QUEUE, PROCESSING '" + header("CamelHttpMethod") + "'")
		.process(new Processor() {
			@Override
			public void process(Exchange exchg) throws Exception {
				if (exchg.getIn().getHeaders().containsKey("CamelHttpMethod")) {
					if ("GET".equalsIgnoreCase(exchg.getIn().getHeader("CamelHttpMethod").toString())) {
						if ("getPerson".equalsIgnoreCase(exchg.getIn().getHeader("operationName").toString())) {
							exchg.getOut().setBody(Person.demoItem());
						} else if ("getContentItem".equalsIgnoreCase(exchg.getIn().getHeader("operationName").toString())) {
							exchg.getOut().setBody(ContentItem.demoItem());
						} else if ("getTrinityStatus".equalsIgnoreCase(exchg.getIn().getHeader("operationName").toString())) {
							exchg.getOut().setBody(TrinityStatus.demoItem());
						} else if ("getCanon".equalsIgnoreCase(exchg.getIn().getHeader("operationName").toString())) {
							HashMap<String,Object> canon = new HashMap<String,Object>();
							canon.put("ContentItem", ContentItem.demoItem());
							canon.put("Person", Person.demoItem());
							canon.put("TrinityStatus", TrinityStatus.demoItem());
							exchg.getOut().setBody(canon);
						} else {
							log.error("Request made for unknown canon object: " + exchg.getIn().getHeader("operationName").toString());
							exchg.getOut().setBody(new TrinityStatus("998", 
									"Canon item name '" + 
											exchg.getIn().getHeader("operationName").toString().replaceFirst("get", "") + 
											"' is not found", 
									"Include a valid Canon item name in your request path, e.g. /Person or /ContentItem.  "
									+ "For complete list of valid Trinity canon items see http://localhost:8182/cxf/canon?_wadl", 
									new Exception().getStackTrace()));
							exchg.getOut().setHeader(Exchange.HTTP_RESPONSE_CODE, constant(404));
						}
					} else if ("PUT".equalsIgnoreCase(exchg.getIn().getHeader("CamelHttpMethod").toString())) {
						exchg.getOut().setBody(new TrinityStatus("998", 
								"Canon item name '" + 
										exchg.getIn().getHeader("operationName").toString().replaceFirst("get", "") + 
										"' added", 
								"Well done grasshopper!", 
								new Exception().getStackTrace()));
					} else if ("POST".equalsIgnoreCase(exchg.getIn().getHeader("CamelHttpMethod").toString())) {
						exchg.getOut().setBody(new TrinityStatus("998", 
								"Canon item name '" + 
										exchg.getIn().getHeader("operationName").toString().replaceFirst("get", "") + 
										"' updated", 
								"Well done grasshopper!", 
								new Exception().getStackTrace()));
					} else if ("DELETE".equalsIgnoreCase(exchg.getIn().getHeader("CamelHttpMethod").toString())) {
						exchg.getOut().setBody(new TrinityStatus("998", 
								"Canon item name '" + 
										exchg.getIn().getHeader("operationName").toString().replaceFirst("get", "") + 
										"' removed", 
								"Well done grasshopper!", 
								new Exception().getStackTrace()));
					}
				}
				log.info("the response class going out is: " + exchg.getOut().getBody().getClass());
			}
		})
		.log("PROCESSOR DONE PROCESSING REQUEST");
	}
}