package org.trinityhealth.example.canon;

import java.text.SimpleDateFormat;
import java.util.Date;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map.Entry;
import java.util.Random;

import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement
public class ContentItem extends CanonModel {
	private String name;
	private String guid;
	private String author;
	private String create_date;
	private String source_system;
	private HashMap<String,String> actions;

	private static String[] lotr_names = {"Frodo Baggins","Gandalf","Smeagol","Gimli son of Gloin"};
	private static String[] hp_names = {"Harry Potter","Ronald Weasley","Albus Brian Wulfric Dumbledore","Severus Snape"};
	private static String[] narnia_names = {"Aslan","Jadis Queen of Charn","Puddleglum","Aravis Tarkheena"};
	private static String[] authors = {"J.R.R. Tolkien","J.K. Rowling","C.S. Lewis"};
	private static HashMap<String, String[]> names = new HashMap<String, String[]>();
	private static String[] sources = {"Sharepoint","ECM","Jive","Liferay"};
	private static HashMap<String, String> action_urls = new HashMap<String, String>();
	private static Random rand = new Random();
	private static SimpleDateFormat fmt = new SimpleDateFormat("MM/dd/yyyy");
	
	static {
		names.put(authors[0], lotr_names);
		names.put(authors[1], hp_names);
		names.put(authors[2], narnia_names);
		
		action_urls.put("Sharepoint", "http://portal.che.org/Pages/Default.aspx");
		action_urls.put("ECM", "http://ecm.trinity-health.org/ecm/");
		action_urls.put("Jive", "https://uniting.trinity-health.org/login.jspa");
		action_urls.put("Liferay", "http://intranet.trinity-health.org/");
	}
	
    public ContentItem() {
		super();
    	actions = new HashMap<String,String>();
    }
    
    public ContentItem(String name, String guid, String author,	String create_date, String source_system, String action, String action_link, ContentItem documentation) {
		super();
		this.name = name;
		this.guid = guid;
		this.author = author;
		this.create_date = create_date;
		this.source_system = source_system;
		this.actions = new HashMap<String,String>();
		if (action != null && action_link != null) this.actions.put(action, action_link);
		this.setDocumentation(documentation);
	}

   
	public static ContentItem demoItem() {
    	ContentItem this_item = new ContentItem();
    	this_item.author = authors[rand.nextInt(authors.length)];
    	this_item.name = names.get(this_item.author)[rand.nextInt(names.get(this_item.author).length)];
    	this_item.guid = String.valueOf(rand.nextDouble());
    	Date create_date = new Date();
    	create_date.setTime(create_date.getTime() - 1000*60*60*24*rand.nextInt(100));
		this_item.create_date =  fmt.format(create_date);
		this_item.source_system = sources[rand.nextInt(sources.length)];
		this_item.actions.put("view", action_urls.get(this_item.source_system));
		this_item.setDocumentation(new ContentItem("ContentItem Documentation","NCC1701","Scott Cornell","01/01/2000","ECM","view","http://ecm.trinity-health.org", null));
    	return this_item;
    }

	@Override
	public boolean isValid() {
		if (this.name == null || this.name == "") return false;
		if (this.guid == null || this.guid == "") return false;
		if (this.author == null || this.author == "") return false;
		if (this.actions == null) return false;
	    Iterator<Entry<String, String>> it = this.actions.entrySet().iterator();
	    boolean no_view_action = true;
	    while (it.hasNext()) {
	    	Entry<String, String> action = it.next();
	    	if (action.getKey().equalsIgnoreCase("view")) no_view_action = false;
	    	if (!action.getValue().startsWith("http")) return false;
	    }
	    if (no_view_action) return false;
	    return true;
	}
	
	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	public String getGuid() {
		return guid;
	}

	public void setGuid(String guid) {
		this.guid = guid;
	}

	public String getAuthor() {
		return author;
	}

	public void setAuthor(String author) {
		this.author = author;
	}

	public String getCreate_date() {
		return create_date;
	}

	public void setCreate_date(String create_date) {
		this.create_date = create_date;
	}

	public HashMap<String, String> getActions() {
		return actions;
	}

	public void setActions(HashMap<String, String> actions) {
		this.actions = actions;
	}

	public String getSource_system() {
		return source_system;
	}

	public void setSource_system(String source_system) {
		this.source_system = source_system;
	}
}
