package org.trinityhealth.example.canon;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;

import java.lang.reflect.Constructor;

import org.junit.Test;

public class CanonItemTester {
	private Class<?> the_class;
	
    @Test
    public void testZeroArgumentConstructor() {
		Constructor<?> zero_arg_ctor = null;
		try {
			assertNotNull("No canonical item selected", the_class);
			Constructor<?>[] ctors = (Constructor<?>[]) the_class.getDeclaredConstructors();
			assertTrue("Wrong number of constructors: " + ctors.length, ctors.length == 2); 

			for (int i = 0; i < ctors.length; i++) {
				zero_arg_ctor = ctors[i];
				if (zero_arg_ctor.getParameterTypes().length == 0)
					break;
			}
			assertNotNull("No 0 argument constructor found in class " + the_class.getName(), zero_arg_ctor);

			Object new_canon_item = null;
			try {
				new_canon_item = zero_arg_ctor.newInstance();
			} catch (Exception e) {
				assertTrue("0 argument constructor for class " + the_class.getName() + " failed: " + e.toString(), false);
			}
			assertNotNull("0 argument constructor for class " + the_class.getName() + " returns nothing", new_canon_item);
			assertTrue("0 argument constructor for class " + the_class.getName() + " does not return a CanonModel child", CanonModel.class.isInstance(new_canon_item));
			assertFalse("0 argument constructor for class " + the_class.getName() + "should not create a valid instance", ((CanonModel)new_canon_item).isValid());
		} catch (Exception e) {
			assertTrue("Problem testing " + the_class.getName() + ": " + e.toString(), true);
		}
	};
	
    @Test
    public void testFieldConstructor() {
		Constructor<?> field_ctor = null;
		try {
			assertNotNull("No canonical item selected", the_class);
			
			Constructor<?>[] ctors = (Constructor<?>[]) the_class.getDeclaredConstructors();
			assertTrue("Wrong number of constructors: " + ctors.length, ctors.length == 2); 
			
			for (Constructor<?> ctor : ctors) {
				if (ctor.getGenericParameterTypes().length != 0) {
					field_ctor = ctor;
					break;
				}
			}
			assertNotNull("No all fields constructor found in class " + the_class.getName(), field_ctor);
			
			Object[] initargs = new Object[field_ctor.getParameterTypes().length];
			for (int i = 0; i < field_ctor.getParameterTypes().length; i++) {
				try {
					initargs[i] = initConstructorParameter(field_ctor.getParameterTypes()[i]);
				} catch (Exception e) {
				}
			}

			Object new_canon_item = null;
			try {
				new_canon_item = field_ctor.newInstance(initargs);
			} catch (Exception e) {
				assertTrue("all fields constructor for class " + the_class.getName() + " failed: " + e.toString(), false);
			}
			assertNotNull("all fields constructor for class " + the_class.getName() + " returns nothing", new_canon_item);
			assertTrue("all fields constructor for class " + the_class.getName() + " does not return a CanonModel child", CanonModel.class.isInstance(new_canon_item));
		} catch (Exception e) {
			assertTrue("Problem testing " + the_class.getName() + ": " + e.toString(), false);
		}
    };
    
    @Test
    public void testDemoItem() {
    	assertNotNull("No canonical item selected", the_class);
		
		Object the_instance = null;
		try {
			the_instance = the_class.newInstance();
		} catch (Exception e1) {
			assertFalse("Unable to instansiate class " + the_class.getName() + ": " + e1.toString(), true);
		}
		
		CanonModel demo_item = null;
		if (the_instance != null) {
			try {
				demo_item = (CanonModel) the_class.getMethod("demoItem").invoke(the_instance);
			} catch (Exception e) {
				assertFalse("Call to demoItem method on class " + the_class.getName() + " fails: " + e.toString(), true);
			}
		}
		
		assertTrue("demoItem() does not return an instance of " + the_class.getName(), the_class.isInstance(demo_item));
		assertTrue("demoItem() returns invalid instance of " + the_class.getName(), demo_item.isValid());
    };

	public Class<?> getThe_class() {
		return the_class;
	}

	public void setThe_class(Class<?> the_class) {
		this.the_class = the_class;
	}

	private Object initConstructorParameter(Class<?> clazz) {
		Object result = null;
		
		try {
			result = clazz.newInstance();
		} catch (Exception e) {}
		
		if (clazz.isPrimitive()) {
			switch (clazz.getName()) {
			case "byte": 	
				return new Byte((byte) 0);
			case "short":
				return new Short((short) 0);
			case "int":
				return new Integer(0);
			case "long":
				return new Long(0);
			case "float":
				return new Float(0);
			case "double":
				return new Double(0);
			case "char":
				return new Character('a');
			case "boolean":
				return new Boolean(false);
			default: 
				return new Integer(0);
			}
		} else if (String.class.isInstance(result)) { 
			return "DUMMY";
		} else {
			return result;
		}
	}
}
