package org.trinityhealth.example.canon;

import java.io.BufferedReader;
import java.io.BufferedWriter;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.FileWriter;
import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;

import org.apache.camel.Exchange;
import org.apache.camel.model.ProcessorDefinition;
import org.apache.camel.test.blueprint.CamelBlueprintTestSupport;
import org.junit.Test;
import org.apache.cxf.jaxws.JaxWsProxyFactoryBean;

public class TestCanonRoute extends CamelBlueprintTestSupport  {
    // retrieve main blueprint descriptor and place in test classpath to make available for test runs
	@Override
	protected String getBlueprintDescriptor() {
		// replicate current version of route descriptor to test area
		try {
			BufferedReader in = new BufferedReader(new FileReader(new File("./src/main/resources/OSGI-INF/blueprint/blueprint.xml")));
			File out_file = new File("./target/test-classes/OSGI-INF/blueprint/blueprint.xml");
			out_file.getParentFile().mkdirs();
			BufferedWriter out = new BufferedWriter(new FileWriter(out_file));
			String in_line;
			while ((in_line=in.readLine())!=null) {
				out.write(in_line);
			}
			out.flush();
			out.close();
			in.close();
		} catch (FileNotFoundException e) {
			return "BAD BLUEPRINT FILE";
		} catch (IOException e) {
			return "BAD BLUEPRINT FILE";
		}
		
		// tell test harness about route descriptor - since test runs in maven test scope, 
		// classpath URI points to /test/resources/... version of the descriptor just created 
		return "OSGI-INF/blueprint/blueprint.xml";
	}
    
    @Test
    public void testGETPerson() throws Exception {
        // additional test to ensure correct endpoints in registry
        assertNotNull("No input endpoint for processor found",context.hasEndpoint("direct:incomingRequestQueue"));

/**   <=== THIS FAILS DUE TO TIMEOUT WAITING FOR CXFRS  ===>     
        // create a client for a person request 
        JaxWsProxyFactoryBean factory = new JaxWsProxyFactoryBean();
        factory.setServiceClass(CanonService.class);
        factory.setAddress("http://localhost:8678/cxf/canon/model/Person");
        CanonService client = (CanonService) factory.create();
        
        // send a message
        Object response = client.getPerson();
**/        
        // send a message directly to processor route, bypassing CXF listener
        HashMap<String,Object> hdrs = new HashMap<String,Object>();
        hdrs.put("CamelHttpMethod", "GET");
        hdrs.put("operationName", "getPerson");
        
        Object response = template.requestBodyAndHeaders("direct:incomingRequestQueue", "", hdrs);
        assertTrue("GET /Person did not return a Person object: " + response.getClass().getName(), Person.class.isInstance(response));
        assertTrue("GET /Person did not return a valid Person object", ((Person)response).isValid());
    }
    
    @Test
    public void testGETContentItem() throws Exception {
        // additional test to ensure correct endpoints in registry
        assertNotNull("No endpoint for processor found",context.hasEndpoint("direct:incomingRequestQueue"));
        
        // send a message directly to processor route, bypassing CXF listener
        HashMap<String,Object> hdrs = new HashMap<String,Object>();
        hdrs.put("CamelHttpMethod", "GET");
        hdrs.put("operationName", "getContentItem");
        
        Object response = template.requestBodyAndHeaders("direct:incomingRequestQueue", "", hdrs);
        assertTrue("GET /ContentItem did not return a ContentItem object: " + response.getClass().getName(), ContentItem.class.isInstance(response));
        assertTrue("GET /ContentItem did not return a valid ContentItem object", ((ContentItem)response).isValid());
    }
    
    @Test
    public void testGETTrinityStatus() throws Exception {
        // additional test to ensure correct endpoints in registry
        assertNotNull("No endpoint for processor found",context.hasEndpoint("direct:incomingRequestQueue"));
        
        // send a message directly to processor route, bypassing CXF listener
        HashMap<String,Object> hdrs = new HashMap<String,Object>();
        hdrs.put("CamelHttpMethod", "GET");
        hdrs.put("operationName", "getTrinityStatus");
        
        Object response = template.requestBodyAndHeaders("direct:incomingRequestQueue", "", hdrs);
        assertTrue("GET /TrinityStatus did not return a TrinityStatus object: " + response.getClass().getName(), TrinityStatus.class.isInstance(response));
        assertTrue("GET /TrinityStatus did not return a valid TrinityStatus object", ((TrinityStatus)response).isValid());
    }
    
    @SuppressWarnings("unchecked")
	@Test
    public void testModel() throws Exception {
        // additional test to ensure correct endpoints in registry
        assertNotNull("No endpoint for processor found",context.hasEndpoint("direct:incomingRequestQueue"));
        
        // send a message directly to processor route, bypassing CXF listener
        HashMap<String,Object> hdrs = new HashMap<String,Object>();
        hdrs.put("CamelHttpMethod", "GET");
        hdrs.put("operationName", "getCanon");
        
        Object response = template.requestBodyAndHeaders("direct:incomingRequestQueue", "", hdrs);   
        assertTrue("GET / did not return a HashMap of objects: " + response.getClass().getName(), HashMap.class.isInstance(response));
        
        Iterator<String> it = ((HashMap<String,CanonModel>)response).keySet().iterator();
        while (it.hasNext()) {
        	Object canon_obj = ((HashMap<String,CanonModel>)response).get(it.next());
            assertTrue("GET / returned an object which is not a CanonModel object " + canon_obj.getClass().getName(), CanonModel.class.isInstance(canon_obj));
            assertTrue("GET / returned a CanonModel object that is not valid " + canon_obj.getClass().getName(), CanonModel.class.isInstance(canon_obj));
		}
        
        assertTrue("GET / returned the wrong number of CanonModel objects", ((HashMap<String,CanonModel>)response).size() == 3);
    }
    
    @Override
    public boolean isUseDebugger() {
        return true;
    }
 
    @Override
    protected void debugBefore(Exchange exchange, org.apache.camel.Processor processor, ProcessorDefinition<?> definition, String id, String label) {
        log.info("Before " + definition);
		log.info("the request headers are: " + exchange.getIn().getHeaders().toString());
		log.info("the body class is: " + exchange.getIn().getBody().getClass());
		log.info("the body content is: " + exchange.getIn().getBody().toString());
    }
 
    @Override
    protected void debugAfter(Exchange exchange, org.apache.camel.Processor processor, ProcessorDefinition<?> definition, String id, String label, long timeTaken) {
        log.info("After " + definition);
		log.info("the request headers are: " + exchange.getIn().getHeaders().toString());
		log.info("the body class is: " + exchange.getIn().getBody().getClass());
		log.info("the body content is: " + exchange.getIn().getBody().toString());
    }}
