package org.trinityhealth.example.canon;

import static org.junit.Assert.assertTrue;
import static org.junit.Assert.assertNotNull;

import java.util.HashMap;
import java.util.Iterator;

import org.junit.Test;

public class TestCanonService {
	private CanonService the_service = new CanonService();
	
    @Test
    public void testGETAll() {
    	HashMap<?, ?> the_canon = (HashMap<?, ?>) the_service.getEntireCanon();
    	assertNotNull("service" + the_service.getClass().getName() + " returns null object for the canon", the_canon);
		assertTrue("service" + the_service.getClass().getName() + " did not return a HashMap of CanonModel objects", HashMap.class.isInstance(the_canon));

    	try {
			Iterator<?> it = ((HashMap<?,?>)the_canon).keySet().iterator();
			while (it.hasNext()) {
				Object key = it.next();
				assertTrue("invalid canon name key found: " + key.toString() + "(" + key.getClass().getName() + ")", String.class.isInstance(key));
				Object canon_item = ((HashMap<?,?>)the_canon).get(key);
				assertTrue("object named " + key + " is not a CanonModel object", CanonModel.class.isInstance(canon_item));
				try {
					assertTrue("object named " + key + " is not a org.trinityhealth.example.canon." + key + " object ", 
							Class.forName("org.trinityhealth.example.canon." + key).isInstance(canon_item));
				} catch (ClassNotFoundException e) {
					assertTrue("object name in canon can not be created as org.trinityhealth.example.canon." + key,false);
				}
				assertTrue("object named " + key + " is not valid", ((CanonModel)canon_item).isValid());
			}
		} catch (ClassCastException e) {
			assertTrue("service does not return a HashMap of potential canonical objects",false);
		}
    }    	

    @Test
    public void testGETOne() {
    	String[] canon_items = {"Person","ContentItem","TrinityStatus"};
    	for (int i = 0; i < canon_items.length; i++) {
    		Class<?> the_item_class = null;
    		try {
				the_item_class = Class.forName("org.trinityhealth.example.canon." + canon_items[i]);
			} catch (ClassNotFoundException e) {
	    		assertTrue("can't find class org.trinityhealth.example.canon." + canon_items[i], false);
			}
    		
        	CanonModel the_canon_item = the_service.getCanonItem(canon_items[i]);
    		assertNotNull("service" + the_service.getClass().getName() + " returns nothing for item " + canon_items[i], the_canon_item);
    		
    		if (the_item_class != null) {
	    		assertTrue("GET /" +canon_items[i]+ " to service" + the_service.getClass().getName()  + " returns " + the_canon_item + ", not " + the_item_class.getName(), the_item_class.isInstance(the_canon_item));
	    		assertTrue("GET /" +canon_items[i]+ " to service" + the_service.getClass().getName()  + " returns an invalid " + the_item_class.getName(), the_canon_item.isValid());
    		}
		}
	}

    @Test
    public void testGETInvalid() {
    	CanonModel the_canon_item = the_service.getCanonItem("***INVALIDITEM***");
		assertNotNull("service" + the_service.getClass().getName() + " returns nothing for invalid item search", the_canon_item);
		
    	assertTrue("GET /***INVALIDITEM*** to service " + the_service.getClass().getName()  + " returned something other than a status message: " + the_canon_item.getClass().getName(), 
    			TrinityStatus.class.isInstance(the_canon_item));
    }

    @Test
    public void testGETMissing() {
    	CanonModel the_canon_item = the_service.getCanonItem(null);
		assertNotNull("service" + the_service.getClass().getName() + " returns nothing for null item search", the_canon_item);
		
    	assertTrue("GET null to service " + the_service.getClass().getName()  + " returned something other than a status message" + the_canon_item.getClass().getName(), 
    			TrinityStatus.class.isInstance(the_canon_item));
    }
}
